"""
File with all exceptions used on Paranoid Model
"""


class SoftDeleted(Exception):
    """Object has been soft deleted."""
    pass


class IsNotSoftDeleted(Exception):
    """Object has not been soft deleted."""
    pass


"""
File with a based Paranoid model
"""


from django.db import models, router
from django.utils import timezone
from django.contrib.admin.utils import NestedObjects
from paranoid_model.exceptions import SoftDeleted, IsNotSoftDeleted
from paranoid_model.manager import ParanoidManager


class Paranoid(models.Model):
    """
    Abstract model with a Paranoid behavior, therefore once deleted
    the instance is just hide from the users: Soft Delete.
    Attributes:
        created_at: DateTimeField
        updated_at: DateTimeField
        deleted_at: DateTimeField
    Properties:
        is_soft_deleted: bool
    """
    SoftDeleted = SoftDeleted
    IsNotSoftDeleted = IsNotSoftDeleted
    objects = ParanoidManager()

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    deleted_at = models.DateTimeField(null=True, blank=True, default=None)

    class Meta:
        abstract = True

    @property
    def is_soft_deleted(self):
        """
        Property to check is current instance has been soft deleted
        Returns:
            bool: if instance has been soft deleted
        """
        return self.deleted_at is not None

    def delete(self, using=None, keep_parents=False, hard_delete=False):
        """
        Override default delete method so Soft Delete can be made.
        If delete is hard_delete, the soft delete is ignored and the
        instance is deleted from the database
        Args:
            using: default None
            keep_parents: default False
            hard_delete: boolean default False
        """
        if not hard_delete:
            self.deleted_at = timezone.now()
            self.save()

            for instance in self._related_objects():
                instance.delete()
        else:
            super(Paranoid, self).delete(using=using, keep_parents=keep_parents)

    def restore(self):
        """
        Restore an instance once deleted and instance's related objects
        """
        self.deleted_at = None
        self.save()

        for related in self._related_objects():
            if 'deleted_at' in related.__dict__.keys():
                related.deleted_at = None
                related.save()

    def _related_objects(self):
        """
        Method to get all objects with foreign key the relation with self instance
        Args:
            self
        Returns:
            List(): all related objects
        """
        collector = NestedObjects(using=router.db_for_write(self))
        collector.collect([self])

        def parse_list(obj):
            """
            Parse to nested objects
            Args:
                obj: list instance or models object instance
            Returns:
                list(): list with objects
            """
            if isinstance(obj, list):
                array = []
                for item in obj:
                    array += parse_list(item)
                return array
            return [obj]

        colletion = parse_list(collector.nested())
        colletion.remove(self)

        return colletion



from django.db import models
from paranoid_model import models as paranoid_model


class Person(paranoid_model.Paranoid):
    """
    Person model with Paranoid inheritance
    Attributes:
         name: CharField
         created_at: DateTimeField
         updated_at: DateTimeField
         deleted_at: DateTimeField
    """
    name = models.CharField(max_length=255)


class Phone(paranoid_model.Paranoid):
    """
    Phone model with Paranoid inheritance
    Attributes:
         phone: CharField
         owner: ForeignKey to Person
         created_at: DateTimeField
         updated_at: DateTimeField
         deleted_at: DateTimeField
    """

    phone = models.CharField(max_length=255)
    owner = models.ForeignKey(Person, on_delete=models.CASCADE, related_name='phones')


class Address(paranoid_model.Paranoid):
    """
    Address model with Paranoid inheritance
    Attributes:
         street: CharField
         owner: ForeignKey to Person
         created_at: DateTimeField
         updated_at: DateTimeField
         deleted_at: DateTimeField
    """
    street = models.CharField(max_length=255)
    owner = models.ForeignKey(Person, on_delete=models.CASCADE, related_name='addresses')



"""
File with QuerySet used on Paranoid Model
"""


from django.db import models
from paranoid_model.exceptions import SoftDeleted, IsNotSoftDeleted


class ParanoidQuerySet(models.query.QuerySet):
    """
    QuerySet for a Paranoid Model with field ``deleted_at`` as a mask
    """

    def get(self, *args, **kwargs):
        """
        Override default behavior of Django's get() to apply a custom validation
        Args:
             *args: passed to Django's get
             **kwargs: passed to Django's get
        Returns:
            Object, instance of object not soft deleted or not hard deleted
        Raise:
            model.DoesNotExist: object not found on database
            paranoid_model.SoftDeleted: object has been soft deleted
            model.MultipleObjectsReturned: if filtered more than 1 instance
        """
        objeto = super(ParanoidQuerySet, self).get(*args, **kwargs)

        if objeto.is_soft_deleted:
            raise SoftDeleted(
                "Object %s has been soft deleted. Try use get_deleted() or get_or_restore()." %
                self.model._meta.object_name)

        return objeto
    
    def get_deleted(self, *arg, **kwargs):
        """
        Method to get a instance that has not been soft deleted yet.
        Args:
             *args: passed to Django's get
             **kwargs: passed to Django's get
        Returns:
            Object: instance of object not soft deleted and not hard deleted
        Raise:
            model.DoesNotExist: object not found on database
            paranoid_model.IsNotSoftDeleted: object has not been soft deleted yet
            model.MultipleObjectsReturned: if filtered more than 1 instance
        """
        kwargs['deleted_at__isnull'] = False
        objeto = super(ParanoidQuerySet, self).get(*arg, **kwargs)

        if not objeto.is_soft_deleted:
            raise IsNotSoftDeleted(
                "Object %s has not been soft deleted yet. Try get()" %
                self.model._meta.object_name)
        
        return objeto
    
    def get_or_restore(self, *args, **kwargs):
        """
        Method to get an instance, and if has been soft deleted it will be restored
        Args:
             *args: passed to Django's get
             **kwargs: passed to Django's get
        Returns:
            Object: instance of object not soft deleted
        Raise:
            model.DoesNotExist: (Django) object not found on database
            model.MultipleObjectsReturned: (Django) more than 1 instances with matches querry
        """

        objeto = super(ParanoidQuerySet, self).get(*args, **kwargs)
        if objeto.is_soft_deleted:
            objeto.restore()
        
        return objeto

    def all(self, with_deleted=False):
        """"
        Override default behavior of Django's all() to filter only not soft deleted or
        include the soft deleted.
        Args:
            with_deleted: bool to check if filter soft deleted or not. Default {False}
        Returns:
            ParanoidQuerySet[]
        """
        return self.filter(with_deleted=with_deleted)

    def filter(self, with_deleted=True, *args, **kwargs):
        """
        Override default behavior of Django's filter() to filter not sotf deleted or include
        instaces that has been soft_deleted.
        
        ``with_deleted`` has a default True because some Django's features call directly
        this method, like a ManyToMant field with related name, and in that case we want
        to have the default behavior and not be on Django's way. So we assume that 
        every paranoid method that calls this filter() will pass a with_deleted and so 
        work as user expects.

        It is also assumed that a ParanoidQueryset[] has already filtered the instances
        soft deleted according to the param whith_delted and the nested filter() wont need
        to check again, and filter without deleted, like ``objects.filter().filter().filter()``. 
        Onlty the firts filter will need to have ``with_deleted`` param, like: 
        ``objects.filter(with_deleted=False).filter().filter()``

        Args:
            with_deleted: bool to check if filter soft deleted or not. Default {True}.
        Returns:
            ParanoidQuerySet[]
        """

        for key in kwargs.keys():
            if key.startswith('deleted_at'):
                kwargs.pop(key)
                break

            # when related names are used django first query if a filter
            # filtering the objects related and after that django filter
            # with user's filter.
            # When Django filter a object soft deleted, with_deledt should
            # be True.
            elif isinstance(kwargs[key], models.Model) and not with_deleted:
                if kwargs[key].is_soft_deleted:
                    with_deleted = True
        
        if not with_deleted:
            kwargs['deleted_at__isnull'] = True
        return super(ParanoidQuerySet, self).filter(*args, **kwargs)

    def delete(self, hard_delete=False):
        """
        Delet instances from current QuerySet
        Args:
            hard_delete: bool to check if apply soft delete or django's delete
        Returns:
            int(): amount deleted
        """

        if not hard_delete:
            cont = 1
            for instance in self:
                instance.delete()
                cont += 1
            # Clear the result cache, in case this QuerySet gets reused.
            self._result_cache = None
            return cont
        else:
            return len(super(ParanoidQuerySet, self).delete())
    
    def restore(self):
        """
        Restore instances from current QuerySet
        
        Returns:
            int(): amount restored
        """

        cont = 1
        for instance in self:
            if instance.is_soft_deleted:
                instance.restore()
                cont += 1
        # Clear the result cache, in case this QuerySet gets reused.
        self._result_cache = None
        return cont

    def deleted_only(self):
        """
        Filter only deleted instances
        Returns:
            ParanoidQuerySet[]
        """
        return self.exclude(deleted_at__isnull=True)



from faker import Faker
from paranoid_model.tests.models import Person, Phone, Address

"""
File wiht utils functions
"""


fake = Faker('en_US')


def isdeleted(obj):
    """
    Function to check if model instance is deleted
    Args:
        obj: instance os a paranoid Model
    Returns:
        bool(): if obj is soft deleted
    """
    return obj.deleted_at is not None


def all_list(iterable, key):
    """
    Function to check if all item in iterable pass the key
    Args:
        iterable: iterable with all objects
        key: function to apply on every object
    Returns:
        bool(): if all objects pass the key
    """
    return all([key(item) for item in iterable])


def any_list(iterable, key):
    """
    Function to check if all item in iterable pass the key
    Args:
        iterable: iterable with all objects
        key: function to apply on every object
    Returns:
        bool(): if any objects pass the key
    """
    return any([key(item) for item in iterable])


def get_person_instance():
    """
    Function to return an instance of a Person
    not saved on database
    Returns:
        Person()
    """
    return Person(name=fake.prefix() + fake.name())


def create_list_of_person(quantity_saved, quantity_deleted):
    """
    Function to create a list of person on database
    Args:
        quantity_saved: int with amount of instances to save
        quantity_deleted: int with amount of instances to save and then delete
    """
    for count in range(quantity_saved):
        person = get_person_instance()
        person.save()

    for count in range(quantity_deleted):
        person = get_person_instance()
        person.save()
        person.delete()


def get_phone_instance(owner):
    """
    Function to return an instance of a Phone
    not saved on database
    Returns:
        Phone()
    """
    return Phone(phone=fake.phone_number(), owner=owner)

def get_address_instance(owner):
    """
    Function to return an instance of a Address
    not saved on database
    Returns:
        Address()
    """
    return Address(street=fake.address(), owner=owner)


from django.test import TestCase
from paranoid_model.tests.models import Person, Phone
from faker import Faker
from paranoid_model.tests.utils import (
    any_list, all_list, get_person_instance, create_list_of_person,
    get_phone_instance, get_address_instance
)


class RelatedModelTest(TestCase):
    """Test model with relatioships ManyToMany, ForeignKey, OneToOne"""
    
    def setUp(self):
        pass

    def assertNotRaises(self, function):
        """
        Method to check if a function does not raises an exception
        Args:
            function: callback function name
        """
        try:
            function()
        except Exception as exc:
            self.fail('Raised in a query that was not suposed to! Message: {}'.format(exc))

    def test_create(self):
        """Test creation of a related model"""
        person = get_person_instance()
        person.save()
        get_phone_instance(person).save()

        all_phones = person.phones.all()
        self.assertEquals(all_phones.count(), 1)
    
    def test_delete_cascade(self):
        """Test delete with cascade"""
        person = get_person_instance()
        person.save()

        phone1 = get_phone_instance(person)
        phone1.save()
        
        person.delete()

        self.assertNotRaises(lambda: person.phones.get_deleted(owner=person))
        phone1 = person.phones.get_deleted(owner=person)
        
        self.assertTrue(person.is_soft_deleted and phone1.is_soft_deleted)
    
    def test_delete_cascade_with_many_objects(self):
        """Test delete cascade with many objects"""
        person = get_person_instance()
        person.save()

        for counter in range(20):
            get_phone_instance(person).save()
            get_address_instance(person).save()
        
        all_phones = person.phones.all()
        self.assertEquals(all_phones.count(), 20)

        person.delete()
        all_phones = person.phones.all(with_deleted=True)
        all_phones_without_deleted = all_phones.all()

        self.assertEquals(all_phones.count(), 20)
        self.assertEquals(all_phones_without_deleted.count(), 0)

        all_address = person.addresses.all(with_deleted=True)
        all_address_without_deleted = all_address.all()
        self.assertEquals(all_address.count(), 20)
        self.assertEquals(all_address_without_deleted.count(), 0)

    def test_restore_cascade(self):
        """Test restore cascade"""
        person = get_person_instance()
        person.save()

        for counter in range(20):
            get_phone_instance(person).save()
            get_address_instance(person).save()
        
        person.delete()
        self.assertEquals(person.phones.all(with_deleted=False).count(), 0)
        self.assertEquals(person.addresses.all(with_deleted=False).count(), 0)

        person.restore()
        self.assertEquals(person.phones.all(with_deleted=False).count(), 20)
        self.assertEquals(person.addresses.all(with_deleted=False).count(), 20)

    def test_restore_cascade_in_queryset(self):
        """Test restore on cascade in a queryset.restore()"""
        amount, amount_phones = 20, 3
        person_list = [get_person_instance() for counter in range(amount)]
        for person in person_list:
            person.save()
            for counter in range(amount_phones):
                get_phone_instance(person).save()
            person.delete()

        Person.objects.all(with_deleted=True).restore()

        person_all = Person.objects.all()
        self.assertEquals(person_all.count(), amount)
        
        for person in person_all:
            self.assertFalse(person.is_soft_deleted)
            self.assertEquals(person.phones.all().count(), amount_phones)

    def test_related_name_queries_all(self):
        """Test related name query .all()"""
        person = get_person_instance()
        person.save()
        
        phone1 = get_phone_instance(person)
        phone1.save()
        phone2 = get_phone_instance(person)
        phone2.save()

        self.assertEquals(person.phones.all().count(), 2)

        phone1.delete()
        self.assertEquals(person.phones.all().count(), 1)
        self.assertEquals(person.phones.all(with_deleted=True).count(), 2)
        
        phone1.restore()
        person.delete()
        self.assertEquals(person.phones.all().count(), 2)
        self.assertEquals(person.phones.all(with_deleted=True).count(), 2)
        self.assertEquals(person.phones.all(with_deleted=False).count(), 0)
    
    def test_related_name_queries_filter(self):
        """Test related name query .filter()"""
        person = get_person_instance()
        person.save()
        
        phone1 = get_phone_instance(person)
        phone1.save()
        phone2 = get_phone_instance(person)
        phone2.save()
        
        phone1.delete()
        self.assertEquals(person.phones.filter(owner=person).count(), 1)
        self.assertEquals(person.phones.filter(owner=person, with_deleted=True).count(), 2)
        self.assertEquals(person.phones.filter(owner=person, with_deleted=False).count(), 1)
    
    def test_get_on_related(self):
        """Test .get() wiht related_name query"""

        person = get_person_instance()
        person.save()

        phone1 = get_phone_instance(person)
        phone1.save()

        self.assertNotRaises(lambda: person.phones.get(phone=phone1.phone))
        self.assertRaises(
            Phone.DoesNotExist, 
            lambda: person.phones.get(phone=phone1.phone+'0'))
        
        phone1.delete()
        self.assertRaises(
            Phone.SoftDeleted, 
            lambda: person.phones.get(phone=phone1.phone))
        
        self.assertRaises(
            Phone.DoesNotExist, 
            lambda: person.phones.get(phone=phone1.phone+'0'))

    def test_get_deleted(self):
        """Test .get_deleted() wiht related_name query"""

        person = get_person_instance()
        person.save()

        phone1 = get_phone_instance(person)
        phone1.save()
        phone2 = get_phone_instance(person)
        phone2.save()
        phone2.delete()

        self.assertRaises(
            Phone.IsNotSoftDeleted,
            lambda: person.phones.get_deleted(phone=phone1.phone))

        self.assertNotRaises(
            lambda: person.phones.get_deleted(phone=phone2.phone))
        
        self.assertRaises(
            Phone.MultipleObjectsReturned,
            lambda: person.phones.get(owner=person))
        
    def test_get_or_restore(self):
        """Test get_or_restore() related_name query"""

        person = get_person_instance()
        person.save()

        phone1 = get_phone_instance(person)
        phone1.save()
        phone1.delete()

        self.assertFalse(person.phones.get_or_restore(phone=phone1.phone).is_soft_deleted)

        get_phone_instance(person).save()
        self.assertRaises(
            Phone.MultipleObjectsReturned,
            lambda: person.phones.get_or_restore())
        
        self.assertRaises(
            Phone.DoesNotExist,
            lambda: person.phones.get_or_restore(phone='a'))

    def test_filter_deleted_only(self):
        """Test .deleted_only() with related name queries"""
        
        person = get_person_instance()
        person.save()

        for counter in range(100):
            phone = get_phone_instance(person)
            phone.save()

            if counter % 2 == 0:
                phone.delete()
        
        deleted = person.phones.deleted_only()
        self.assertEquals(deleted.count(), 50)

        deleted_zero = person.phones.all().deleted_only()
        self.assertEquals(deleted_zero.count(), 0)



import os

SECRET_KEY = 'fake-key'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'paranoid_model.tests'
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3'
    }
}



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='django-paranoid-model', # Replace with your own username
    version='0.0.1',
    url='https://github.com/DarknessRdg/django-paranoid-model',
    keywords='django paranoid safedelete softdelete',

    author='Luan Rodrigues',
    author_email='luanrodriguesbusiness@hotmail.com',
    
    description='Django abstract model with paranoid behavior,',
    long_description=long_description,
    long_description_content_type='text/markdown',
    
    packages=setuptools.find_packages(),
    
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],

    install_requires=['Django'],
    extra_requires={
        'test': ['Faker']
    },
    python_requires='>=3.4',

    project_urls={
        'Bug Reports': 'https://github.com/DarknessRdg/django-paranoid-model/issues',
        'Source': 'https://github.com/DarknessRdg/django-paranoid-model/'
    }
)


"""
File with Manager used on Paranoid Model
"""


from django.db import models, router
from paranoid_model.queryset import ParanoidQuerySet


class ParanoidManager(models.Manager):
    """Paranoid Manager with a Paranoid behavior"""

    _queryset_class = ParanoidQuerySet

    def get_queryset(self):
        return self._queryset_class(self.model, using=self._db)

    def all(self, **kwargs):
        """
        Intercept Django's query all() and use Paranoid Queryset method all()
        Args:
            with_deleted: bool to check if also wants to filter soft deleted instances
        Returns:
            ParanoidQuerySet[]
        """
        try:
            with_deleted = kwargs['with_deleted']
        except KeyError:
            with_deleted = False
        
        try:
            # When Manager has a instace, is need to check if
            # the isnstance has been soft delete. Because in case
            # that instace has been soft delete, the query must be 
            # with deleted and if user has not passad a with_deleted param
            if self.instance.is_soft_deleted and 'with_deleted' not in kwargs.keys():
                with_deleted = True
        except AttributeError:
            pass
        qs = self.get_queryset()
        return qs.all(with_deleted=with_deleted)
    
    def filter(self, with_deleted=False, *args, **kwargs):
        """
        Intercept firts method ``objects.filter()``.
        This is because a querry from related_name doesn't call this 
        method, in that case it can be treated on queryset.filter and know
        when is a related_name query or an objects query.
        
        Args:
            with_deleted: bool to check if filter soft deleted or not. Default {False}
        Returns:
            ParanoidQuerySet[]
        """
        qs = self.get_queryset()
        return qs.filter(with_deleted=with_deleted, *args, **kwargs)
    
    def deleted_only(self):
        """
        Method to filter only deleted instances
        Return:
            ParanoidQuerySet[]
        """

        qs = self.get_queryset()
        return qs.deleted_only()

    def get_deleted(self, *arg, **kwargs):
        """
        Method to get an instance that has not been soft deleted yet.
        Args:
             *args: passed to Django's get
             **kwargs: passed to Django's get
        Returns:
            Object: instance of object not soft deleted and not hard deleted
        Raise:
            model.DoesNotExist: object not found on database
            paranoid_model.IsNotSoftDeleted: object has not been soft deleted yet
            model.MultipleObjectsReturned: if filtered more than 1 instance
        """
        
        return self.get_queryset().get_deleted(*arg, **kwargs)
    
    def get_or_restore(self, *args, **kwargs):
        """
        Method to get a instance, and if has been soft deleted it will be restored
        Args:
             *args: passed to Django's get
             **kwargs: passed to Django's get
        Returns:
            Object: instance of object not soft deleted
        Raise:
            model.DoesNotExist: object not found on database
            model.MultipleObjectsReturned: if filtered more than 1 instance
        """

        return self.get_queryset().get_or_restore(*args, **kwargs)


from django.test import TestCase
from paranoid_model.tests.models import Person
from faker import Faker
from paranoid_model.tests.utils import any_list, all_list, get_person_instance, create_list_of_person


class SingleModelTest(TestCase):
    """Test paranoid model behavion on a sigle model"""

    def setUp(self):
        pass

    def assertNotRaises(self, function):
        """
        Method to check if a function does not raises an exception
        Args:
            function: callback function name
        """
        try:
            function()
        except Exception as exc:
            self.fail('Raised in a query that was not suposed to! Message: {}'.format(exc))

    def test_create(self):
        """Test create of a paranoid model"""
        person = get_person_instance()
        person.save()

        self.assertIsNotNone(person.created_at)
        self.assertIsNotNone(person.updated_at)
        self.assertIsNone(person.deleted_at)

    def test_soft_delete(self):
        """Test delete of a paranoid model"""
        get_person_instance().save()

        person = Person.objects.all().first()
        person.delete()

        self.assertIsNotNone(person.deleted_at)
    
    def test_restore(self):
        """Test restore in a single object"""
        person = get_person_instance()
        person.save()
        
        self.assertFalse(person.is_soft_deleted)

        person.delete()
        self.assertTrue(person.is_soft_deleted)

        person.restore()
        self.assertFalse(person.is_soft_deleted)

    def test_restore_in_a_queryset(self):
        """Test restore in a queryset"""
        save, delete = 100, 0
        create_list_of_person(save, delete)

        Person.objects.all().delete()
        self.assertEquals(Person.objects.all().count(), 0)

        Person.objects.all(with_deleted=True).restore()
        self.assertEquals(Person.objects.all().count(), save)


    def test_soft_delete_in_a_queryset(self):
        """Test delete all instances in a querryset"""
        save, delete = 100, 0
        create_list_of_person(save, delete)

        self.assertTrue(all_list(
            Person.objects.all(with_deleted=True),
            lambda person: not person.is_soft_deleted
        ))

        Person.objects.all().delete()
        self.assertEqual(Person.objects.all().count(), 0)
        self.assertTrue(all_list(
            Person.objects.all(with_deleted=True),
            lambda person: person.is_soft_deleted
        ))

    def test_all(self):
        """Test query all()"""
        save = 100
        delete = 50
        create_list_of_person(save, delete)

        all_without_param = Person.objects.all()
        self.assertTrue(all_list(all_without_param, lambda x: not x.is_soft_deleted))

    def test_all_with_deleted_param(self):
        """Test query all(with_deleted)"""

        save = 100
        delete = 50
        create_list_of_person(save, delete)

        all_without_deleted = Person.objects.all(with_deleted=False)
        self.assertTrue(all_list(all_without_deleted, lambda x: not x.is_soft_deleted))

        all_with_deleted = Person.objects.all(with_deleted=True)
        self.assertTrue(any_list(
            all_with_deleted, 
            lambda x: x.is_soft_deleted))
        
        self.assertTrue(any_list(
            all_with_deleted, 
            lambda x: not x.is_soft_deleted))

    def test_filter(self):
        """Test query filter()"""
        save = 100
        delete = 100
        create_list_of_person(save, delete)

        filter_without_param = Person.objects.filter(name__icontains='.')
        self.assertTrue(all_list(
            filter_without_param, 
            lambda x: not x.is_soft_deleted))
        
        filter_from_filter = filter_without_param.filter(name__icontains='.')
        self.assertTrue(all_list(
            filter_from_filter, 
            lambda x: not x.is_soft_deleted))

    def test_filter_with_deleted_param(self):
        """Test query filter(with_deleted)"""
        save = 100
        delete = 100
        create_list_of_person(save, delete)

        filter_without_deleted = Person.objects.filter(with_deleted=False, name__icontains='.')
        self.assertTrue(all_list(
            filter_without_deleted, 
            lambda x: not x.is_soft_deleted))

        filter_with_deleted = Person.objects.filter(with_deleted=True, name__icontains='.')
        self.assertTrue(any_list(
            filter_with_deleted, 
            lambda x: x.is_soft_deleted))
        
        self.assertTrue(any_list(
            filter_with_deleted, 
            lambda x: not x.is_soft_deleted))

    def test_get(self):
        """Test get an object with Raises and without Raises"""

        person = get_person_instance()
        name = person.name

        person.save()

        self.assertNotRaises(lambda: Person.objects.get(name=name))

        self.assertRaises(
            Person.DoesNotExist, 
            lambda: Person.objects.get(name=name+'^'))
        
        person.delete()
        self.assertRaises(
            Person.DoesNotExist, 
            lambda: Person.objects.get(name=name+'^'))
        
        self.assertRaises(
            Person.SoftDeleted, 
            lambda: Person.objects.get(name=name))
    
    def test_filter_and_get(self):
        """Test .get() after a .filter() query"""

        person = get_person_instance()
        name = person.name
        person.save()

        query = Person.objects.filter(name=name)
        self.assertEquals(query.count(), 1)

        self.assertNotRaises(lambda: query.get(name=name))

        self.assertRaises(
            Person.DoesNotExist, 
            lambda: query.get(name=name+'^'))

        person.delete()
        self.assertRaises(
            Person.DoesNotExist, 
            lambda: query.get(name=name))

        query = Person.objects.filter(name=name, with_deleted=True)
        self.assertRaises(
            Person.SoftDeleted, 
            lambda: query.get(name=name))

    def test_get_deleted(self):
        """Test get on deleted item"""
        
        person = get_person_instance()
        person.save()
        name = person.name 
        
        self.assertRaises(
            Person.IsNotSoftDeleted, 
            lambda: Person.objects.get_deleted(name=name))
        
        person.delete()
        self.assertNotRaises(lambda: Person.objects.get_deleted(name=name))

    def test_get_or_restore(self):
        """Test get_or_restore on an single model"""

        person = get_person_instance()
        person.save()
        person.delete()

        self.assertTrue(person.is_soft_deleted)
        self.assertFalse(Person.objects.get_or_restore(name=person.name).is_soft_deleted)
        
        self.assertRaises(
            Person.DoesNotExist,
            lambda: Person.objects.get_or_restore(name=person.name+'^'))
            
        get_person_instance().save()
        self.assertRaises(
            Person.MultipleObjectsReturned,
            lambda: Person.objects.get())
    
    def test_filter_deleted_only(self):
        """Test filter deleted_only"""
        
        save, delete = 100, 200
        create_list_of_person(save, delete)

        deleted = Person.objects.deleted_only()
        self.assertEquals(deleted.count(), delete)
        
        deleted = Person.objects.all(with_deleted=True).deleted_only()
        self.assertEquals(deleted.count(), delete)

        deleted_zero = Person.objects.all().deleted_only()
        self.assertEquals(deleted_zero.count(), 0)